from tcpgecko import *
import sys, thread, time, re

def make32Uint(addr_str):
	return struct.unpack('>I', addr_str)[0]

def getCmplxAddr(m):
	oaddr = m.group(0)
	if '+' in oaddr:
		oaddr = oaddr.replace('+', ' + ').replace('  ',' ')
	if '-' in oaddr:
		oaddr = oaddr.replace('-', ' - ').replace('  ',' ')
	parts = oaddr.replace('[','').replace(']','').split(' ')
	pint = int(parts[0],16)
	paddr = make32Uint(rpc.readmem(pint,4))
	if '+' in parts:
		paddr += int(parts[2])
	if '-' in parts:
		paddr -= int(parts[2])
	return hex(paddr)[2:]
	
def getSmpleAddr(m):
	oaddr = m.group(0)
	
	pint = int(oaddr.replace('[','').replace(']',''),16)
	paddr = make32Uint(rpc.readmem(pint,4))
	return hex(paddr)[2:]

def resolveCodes(codelist):
	cmplx = re.compile('\[[a-fA-F0-9]{8}[ +-].*?\d{1,3}\]')
	smple = re.compile('\[[a-fA-F0-9]{8}\]')
	resolved_list = []
	print('Resolving...')
	for code in codelist:	
		if code.startswith('['):
			while code.startswith('['):
				print('1st Run!')	
				if '-' in code or '+' in code:
					code = cmplx.sub(getCmplxAddr,code)
					print('complex resolved {}'.format(code))
				code = smple.sub(getSmpleAddr,code)
				print('simple resolved {}'.format(code))
					
		pieces = code.split(' ')
		ptuple = (int(pieces[0],16),int(pieces[1],16))
		resolved_list.append(ptuple)
	return resolved_list


def applyOneTime(codelist):

	for code in codelist:
		addr = code[0]
		val = code[1]
		rpc.pokemem(addr, val)

def applyAlways(codelist,recodelist):
	s = time.time()
	while True:
		for code in codelist:
			addr = code[0]
			const_val = code[1]
			curr_val = make32Uint(rpc.readmem(addr,4))
			if curr_val != const_val:
				rpc.pokemem(addr,const_val)
			e = time.time()
			if int(e-s) == 30:
				codelist = resolveCodes(recodelist)
				s = e

def main():
	#Read Cheatfiles
	infile = open('cheats.txt', 'r').read()
	refile = open('recheats.txt', 'r').read()

	codelist =  re.findall('^(?ims)[\[a-fA-F0-9\] +-]+',infile)
	recodelist = re.findall('^(?ims)[\[a-fA-F0-9\] +-]+',refile)
	
	if len(codelist) > 0:
		codelist = resolveCodes(codelist)
	else:
		print("No codes loaded from cheats.txt, ignoring file")
	if len(recodelist) > 0:
		print("Found {} Codes in recheats.txt".format(len(recodelist)))
		resolvedrecodelist = resolveCodes(recodelist)
	else:
		print("No codes loaded from recheats.txt, ignoring file")	
	
	#Cheats loaded, setup RPC calls
	try:
		if len(codelist) > 0:
			print("Applying {0} One Time Codes...".format(len(codelist)))
			applyOneTime(codelist)
		if len(recodelist) >0:
			print("Applying {0} Constant Value Codes...".format(len(recodelist)))
			applyAlways(resolvedrecodelist,recodelist)
	except Exception as e:
		print(e)

	

			
if __name__ == '__main__':
	rpc = TCPGecko('10.0.0.19')
	print("RPC Connected...\nLoading Codes")
	main()
	
